/*
** Module   : process.c
** Abstract : implementation for process creation/waiting logic in libp2j.so or p2j.dll 
**            native library which is called via JNI
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------Description----------------------
** 001 LMR 20101123 Initial revision of this file. The code was moved from 
**                  Toolkit.c in the charva package (built as libTerminal.so).
**                  It was an original work created by Golden Code and was 
**                  never a part of the Rob Pitman charva distribution.  For
**                  this reason, it is fully owned by Golden Code and has been
**                  moved here to keep it separate from the LGPL charva code.
** 002 GES 20110629 Removed unused var reference that was causing a compile
**                  warning.
** 003 EVL 20130329 Making changes to compile Windows DLL.
*/ 

#include "process_launch.h"

#include "com_goldencode_p2j_util_LaunchManager.h"

// cache field IDs for performance
static jfieldID jfd    = NULL;

/**
 * initializeFileDescriptor - initialize the passed file descriptor object 
 *                            with the pty fd
 *
 * @param   env         pointer to an object providing access to JNI environment

 * @param   descriptor  object representing the FileDescriptor Java instance 
 *
 * @param   fd          the pty file descriptor
 *
 */ 
void initializeFileDescriptor(JNIEnv* env, jobject descriptor, int fd)
{
   if (jfd == NULL)
   {
      jclass fdcls = (*env)->GetObjectClass(env, descriptor);

      jfd = (*env)->GetFieldID(env, fdcls, "fd", "I");
   }

   (*env)->SetIntField(env, descriptor, jfd, fd);
}

/**
 * Java_com_goldencode_p2j_util_LaunchManager_init - 
 *                         set up signal handling to ignore job control
 *                         signals (reflecteds in the child process as well)
 *
 * @param   env   pointer to an object providing access to JNI environment
 *
 * @param   cls   reference to the Java class who called this function
 *                (from static context)
 *
 */ 
JNIEXPORT void JNICALL Java_com_goldencode_p2j_util_LaunchManager_init(
                                                                JNIEnv* env,
                                                                jclass class)
{
   init(env, class);
}

/**
 * Java_com_goldencode_p2j_util_LaunchManager_pseudoTerminalLaunch - 
 *      spawns a child process redirecting its standard input, output and
 *      error to particular file descriptors (FileDescriptor Java instances) 
 *      
 *
 * @param   env       pointer to a structure that provides access to JNI
 *                    environment
 *
 * @param   cls       reference to the Java class who called this function
 *                    (from static context)
 *
 * @param   cmdline   the command line used for launching the child process  
 *
 * @param   in        FileDescriptor Java object for stdin
 *
 * @param   out       FileDescriptor Java object for stdout
 *
 * @param   err       FileDescriptor Java object for stderr
 *
 *
 * @param   silent  boolean flag controlling interactive mode
 *
 * @return  > 1  PID of spawned child process if successful
 *          < 0  on error
 */ 

JNIEXPORT jint JNICALL
   Java_com_goldencode_p2j_util_LaunchManager_pseudoTerminalLaunch(JNIEnv* env,
                                                                   jclass        cls,
                                                                   jobjectArray  cmdline,
                                                                   jobject       in,
                                                                   jobject       out,
                                                                   jobject       err,
                                                                   jboolean      silent)
   
{
   // TODO: Implementation
   return pseudoTerminalLaunch(env, cls, cmdline, in, out, err, silent);
}

/**
 * Java_com_goldencode_p2j_util_LaunchManager_pseudoTerminalWait - 
 *          waits for a child process to terminate. 
 *
 * @param   env     pointer to a structure that provides access to JNI environment
 *
 * @param   cls     reference to the Java class who called this function from
 *                  a static context
 *
 * @param   pid     PID of the process to be waited
 *
 * @param   silent  boolean flag to control interactive mode
 *
 */ 
JNIEXPORT void JNICALL
   Java_com_goldencode_p2j_util_LaunchManager_pseudoTerminalWait(JNIEnv*  env,
                                                                 jclass   cls,
                                                                 jint      pid,
                                                                 jboolean silent)
{
   pseudoTerminalWait(env, cls, pid, silent);
}

